#include<stdio.h>
#include<string.h>

typedef unsigned long  dword;
typedef unsigned short word;
typedef unsigned char  byte;

//  
#define POLYNOM8_1				0xD5
#define POLYNOM8_2				0xAB
#define POLYNOM8_3				0x57
#define POLYNOM8_ATM1			0x07				//(use: ATM HEC)	
#define POLYNOM8_ATM2			0xE0	
#define POLYNOM8_ATM3			0xC1
#define POLYNOM8_CCITT1			0x8D				//(use: 1-Wire bus)
#define POLYNOM8_CCITT2			0xB1
#define POLYNOM8_CCITT3			0x63
#define POLYNOM8_DALLAS1		0x31				//(use: 1-Wire bus)
#define POLYNOM8_DALLAS2		0x8C
#define POLYNOM8_DALLAS3		0x19
#define POLYNOM8_SAE1			0x1D
#define POLYNOM8_SAE2			0xB8

#define POLYNOM16_CCITT1		0x1021				//(X.25, V.41, Bluetooth, PPP, IrDA, BACnet; known as "CRC-CCITT")
#define POLYNOM16_CCITT2		0x8408
#define POLYNOM16_CCITT3		0x0811
#define POLYNOM16_IBM1			0x8005				//(XMODEM, USB, many others; also known as "CRC-16")
#define POLYNOM16_IBM2			0xA001	
#define POLYNOM16_IBM3			0x4003	

#define POLYNOM32_ZIP			0xEDB88320			//(ZIP)
#define POLYNOM32_MPEG2			0x04C11DB7			//(MPEG2)
#define POLYNOM32_MPEG2_1		POLYNOM_ZIP
#define POLYNOM32_MPEG2_2		0xDB710641
#define POLYNOM32_IEEE			POLYNOM_MPEG2 		//(IEEE 802.3)
#define POLYNOM32_IEEE_1		POLYNOM_ZIP
#define POLYNOM32_IEEE_2		POLYNOM_MPEG2_2
#define POLYNOM32_CASTAGNOLI1	0x1EDC6F41 			//(Castagnoli)
#define POLYNOM32_CASTAGNOLI2	0x82F63B78
#define POLYNOM32_Castagnoli3	0x05EC76F1
#define POLYNOM32_KOOPMAN1		0x741B8CD7 			//(Koopman)
#define POLYNOM32_KOOPMAN2		0xEB31D82E

#define POLYNOM64_ISO1			0x000000000000001B	//(ISO 3309)
#define POLYNOM64_ISO2			0xD800000000000000
#define POLYNOM64_ISO3			0xB000000000000001
#define POLYNOM64_ECMA1			0x42F0E1EBA9EA3693	//(ECMA-182)
#define POLYNOM64_ECMA2			0xC96C5795D7870F42
#define POLYNOM64_ECMA3			0x92D8AF2BAF0E1E85



//!     
template<typename T>
class CRC {
	T crc_table[256];
public:
	CRC(T polynom);
	T Calc(byte*,int count);
};


template<typename T>
CRC<T>::CRC(T polynom)
{
	int n,k; T c;
	for(n=0;n<256;n++) {
		c=n;
		for(k=0;k<8;k++) c=c>>1^(-(int)(c&1)&polynom);
		crc_table[n]=c;
	}
}

template<typename T>
T CRC<T>::Calc(byte* data,int count)
{
	T value=(dword)-1; 
	while(count--) value=(value>>8)^crc_table[(byte)value^*data++]; 
	return value; 
}

//     (    5 )
template<typename T>
T _crc(byte *data,int count,T polynom) 
{
	int n;
	T sum=(T)-1;
	while(count--) {
		sum^=*data++;
		for(n=0;n<8;n++) sum=(-(sum&1)&polynom)^(sum>>1);
	}
	return sum;
}



void main()
{
	int size;
	byte data[]="3,1415926535897932384626433832795";
	if(sizeof(dword)!=4 || sizeof(word)!=2 || sizeof(byte)!=1) printf("type error\n\a"),getchar();
	CRC<dword> crc(POLYNOM32_ZIP);
	size=strlen((char*)data);
	if(crc.Calc(data,size)!=_crc(data,size,(dword)POLYNOM32_ZIP)) 
		printf("Internal error\n\a"),getchar();
	printf("node\n");
}

